#ifndef TexturesH //To avoid linker errors, define header's unique "name" so...
#define TexturesH //...this header will not be compilled more than one time
//==============================================================================
#include <gl\gl.h>      // Header file for the main OpenGL library
#include <gl\glu.h>     // Header file for the GLu library
#include <stdlib.h>     // Standart c library
#include <stdio.h>      // File I/O routines
//------------------------------------------------------------------------------

//Auxilary micro-function for rotating red and blue bytes
inline void BGRtoRGB(unsigned char *BGR){
 unsigned char B=BGR[0];
 BGR[0]=BGR[2];
 BGR[2]=B;
}

//Class for managing basic 2D texture; -=< VERSION 1.1 >=-
class TTexture{
//Symbolic texture name, loaded into graphic memory
unsigned int Name;
public:
//Default constructor allow to assign given texture name
inline TTexture(unsigned int TextureName=0):Name(TextureName){}
//Destructor
inline ~TTexture(){if(Name)glDeleteTextures(1,&Name);}
//Function converts 24bit bitmap (*.bmp) to TEX file format (*.tex);
bool BMPtoTEX(const char* FilePath);
//Function loads texture image from *.BMP file and saves it to currently opened
//file as TEX chunk
bool BMPtoChunk(const char* BMPFilePath, FILE* CurrentlyOpenedFile);
//Function loads texture image from *.TEX file and saves it to currently opened
//file as TEX chunk
bool TEXtoChunk(const char* TEXFilePath, FILE* CurrentlyOpenedFile);

//!!!NOTE!!!
//all of the following texture routines can be called only after GL have been
//initialized!!!

//Function loads texture image from windows bitmap file and registers it;
//note that only 24bit-depth bitmaps supported!
bool LoadFromBMP(const char* FilePath);
//Function loads texture from TEX file; it's format extremely simple:
// 4 byte:   unsigned int, width of the image in pixels
// 4 byte:   unsigned int, height of the image in pixels
//(4*width*height) bytes:   pixels data; image' pixels stored row-by-row, each
//                          one in RGBA format (4 unsigned chars per pixel);
bool LoadFromTEX(const char* FilePath);
//Function saves previously loaded texture to TEX file
bool SaveToTEX(const char* FilePath);
//Function loads texture from currently opened file starting to read it from
//current file read/write position and registers texture; after loading done,
//file read/write position will be shifted to the end of texture chunk.
//Such texture chunk must have the folowing format:
// 4 byte:   unsigned int, width of the image in pixels
// 4 byte:   unsigned int, height of the image in pixels
//(4*width*height) bytes:   pixels data; image' pixels stored row-by-row, each
//                          one in RGBA format (4 unsigned chars per pixel);
void LoadFromChunk(FILE* CurrentlyOpenedFile);
//Function saves texture to currently opened file starting to write from
//current file read/write position; after loading done, file read/write position
//will be shifted to the end of texture chunk.
void SaveToChunk(FILE* CurrentlyOpenedFile);
//Function just passes texture chunk without doing anything with it
void PassChunk(FILE* CurrentlyOpenedFile);
//Macros-function just binds this texture to current 2D-texture unit
inline void Bind(){ glBindTexture(GL_TEXTURE_2D,Name); }
//Assignment operator replaces texture ID with specified one
inline void operator = (unsigned int TextureName){ Name=TextureName; }
//Boolean operator compares texture ID with specified one
inline bool operator == (unsigned int TextureName){ return Name==TextureName; }
};

//++++++++++++++++++++++++ TTexture class  definition ++++++++++++++++++++++++\\
bool TTexture::BMPtoTEX(const char* FilePath){
 //Try to open specyfied file; if unsuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"rb"); if(!FileHandler)return false;
 //Pass 18 bytes and read width and height of the image
 fseek(FileHandler,18,0);
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 //Pass two bytes and read bpp
 fseek(FileHandler,2,1);
 unsigned short int bpp;  fread(&bpp,2,1,FileHandler);
 //We handle only RGB (24bpp) bitmap types, so check it
 if(bpp!=24){ fclose(FileHandler); return false; }
 //Pass 24 bytes of stupid header's garbage and read pixels
 fseek(FileHandler,24,1);
 //Count size of row (don't forget about 4-byte-aligning!)
 unsigned long int rowsize = wh[0]*3 + wh[0]%4;
 //Calculate total size of image data
 unsigned long int size = wh[1]*rowsize;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Close bmp file
 fclose(FileHandler);
 //Genenrate new file name
 unsigned int PathLength = strlen(FilePath);
 char* NewName = new char[PathLength+1];
 strcpy(NewName,FilePath);
 NewName[PathLength-3]='t';NewName[PathLength-2]='e'; NewName[PathLength-1]='x';
 //Create new file in the same path with the same name, but with .TEX extension
 FileHandler=fopen(NewName,"wb");if(!FileHandler){delete[] pixels;return false;}
 //Write image dimensions
 fwrite(wh,4,2,FileHandler);
 //Create alternative pixel location to hold converted from BGR to RGBA image
 unsigned int szRGBA = wh[0]*wh[1]*4;
 unsigned char *pixRGBA = new unsigned char[szRGBA];
 //Convert all pixels from BGR to RGBA representation
 for(unsigned int h=0;h<wh[1];h++)for(unsigned int w=0;w<wh[0];w++){
  pixRGBA[h*wh[0]*4+w*4  ] = pixels[h*rowsize+w*3+2];
  pixRGBA[h*wh[0]*4+w*4+1] = pixels[h*rowsize+w*3+1];
  pixRGBA[h*wh[0]*4+w*4+2] = pixels[h*rowsize+w*3];
  pixRGBA[h*wh[0]*4+w*4+3] = 255; //Set alpha-component to 255 explicitly
 }
 //Write converted image to file
 fwrite(pixRGBA,szRGBA,1,FileHandler);
 //Close file and destroy pixels locations
 fclose(FileHandler);
 delete[] pixels;
 delete[] pixRGBA;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
bool TTexture::BMPtoChunk(const char* BMPFilePath, FILE* CurrentlyOpenedFile){
 //Try to open specyfied file; if unsuccesfull, return negative result
 FILE* FileHandler=fopen(BMPFilePath,"rb"); if(!FileHandler)return false;
 //Pass 18 bytes and read width and height of the image
 fseek(FileHandler,18,0);
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 //Pass two bytes and read bpp
 fseek(FileHandler,2,1);
 unsigned short int bpp;  fread(&bpp,2,1,FileHandler);
 //We handle only RGB (24bpp) bitmap types, so check it
 if(bpp!=24){ fclose(FileHandler); return false; }
 //Pass 24 bytes of stupid header's garbage and read pixels
 fseek(FileHandler,24,1);
 //Count size of row (don't forget about 4-byte-aligning!)
 unsigned long int rowsize = wh[0]*3 + wh[0]%4;
 //Calculate total size of image data
 unsigned long int size = wh[1]*rowsize;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Close bmp file
 fclose(FileHandler);
 //Write image dimensions first
 fwrite(wh,4,2,CurrentlyOpenedFile);
 //Create alternative pixel location to hold converted from BGR to RGBA image
 unsigned int szRGBA = wh[0]*wh[1]*4;
 unsigned char *pixRGBA = new unsigned char[szRGBA];
 //Convert all pixels from BGR to RGBA representation
 for(unsigned int h=0;h<wh[1];h++)for(unsigned int w=0;w<wh[0];w++){
  pixRGBA[h*wh[0]*4+w*4  ] = pixels[h*rowsize+w*3+2];
  pixRGBA[h*wh[0]*4+w*4+1] = pixels[h*rowsize+w*3+1];
  pixRGBA[h*wh[0]*4+w*4+2] = pixels[h*rowsize+w*3];
  pixRGBA[h*wh[0]*4+w*4+3] = 255; //Set alpha-component to 255 explicitly
 }
 //Write converted image to file
 fwrite(pixRGBA,szRGBA,1,CurrentlyOpenedFile);
 //Destroy pixels locations
 delete[] pixels;
 delete[] pixRGBA;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
bool TTexture::TEXtoChunk(const char* TEXFilePath, FILE* CurrentlyOpenedFile){
 //Try to open specyfied file; if unsuccesfull, return negative result
 FILE* FileHandler=fopen(TEXFilePath,"rb"); if(!FileHandler)return false;
 //Read image dimensions from TEX file and write them into CurrentlyOpenedFile
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 fwrite(wh,4,2,CurrentlyOpenedFile);
 //If there is no image, stop here (indicating success anyway)
 if(wh[0]==0 || wh[1]==0){ fclose(FileHandler); return true; }
 //Calculate total size of image data
 unsigned long int size = wh[0]*wh[1]*4;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Close file
 fclose(FileHandler);
 //Save image dimensions and pixels as tex-chunk to currently opened file
 fwrite(wh,4,2,CurrentlyOpenedFile);
 fwrite(pixels,size,1,CurrentlyOpenedFile);
 //Destroy pixels location
 delete[] pixels;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
bool TTexture::LoadFromBMP(const char* FilePath){
 //Try to open specyfied file; if unsuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"rb"); if(!FileHandler)return false;
 //Pass 18 bytes and read width and height of the image
 fseek(FileHandler,18,0);
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 //If there is no image, exit here (indicating success anyway)
 if(wh[0]==0 || wh[1]==0){ fclose(FileHandler); return true; }
 //Pass two bytes and read bpp
 fseek(FileHandler,2,1);
 unsigned short int bpp;  fread(&bpp,2,1,FileHandler);
 //We handle only RGB (24bpp) bitmap types, so check it
 if(bpp!=24){ fclose(FileHandler); return false; }
 //Pass 24 bytes of stupid header's garbage and read pixels
 fseek(FileHandler,24,1);
 //Count size of row (don't forget about 4-byte-aligning!)
 unsigned long int rowsize = wh[0]*3 + wh[0]%4;
 //Calculate total size of image data
 unsigned long int size = wh[1]*rowsize;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Swap Red and Green bytes
 //for(unsigned int h=0;h<wh[1];h++)for(unsigned int w=0;w<wh[0];w++)
 // BGRtoRGB(&pixels[h*rowsize+w*3]);
 //Create, load, register texure
 glGenTextures(1,&Name);            //Create new texture name
 glBindTexture(GL_TEXTURE_2D,Name); //Make it current
 //Create mipmaps
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,
                   wh[0],wh[1],
                   32992/*GL_BGR*/,GL_UNSIGNED_BYTE,pixels);
 //Close file and destroy pixels location
 fclose(FileHandler);
 delete[] pixels;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
bool TTexture::LoadFromTEX(const char* FilePath){
 //Try to open specyfied file; if unsuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"rb"); if(!FileHandler)return false;
 //Read image dimensions
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 //If there is no image, exit here (indicating success anyway)
 if(wh[0]==0 || wh[1]==0){ fclose(FileHandler); return true; }
 //Calculate total size of image data
 unsigned long int size = wh[0]*wh[1]*4;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Create, load, register texure
 glGenTextures(1,&Name);            //Create new texture name
 glBindTexture(GL_TEXTURE_2D,Name); //Make it current
 //Create mipmaps
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,
                   wh[0],wh[1],
                   GL_RGBA,GL_UNSIGNED_BYTE,pixels);
 //Close file and destroy pixels location
 fclose(FileHandler);
 delete[] pixels;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
bool TTexture::SaveToTEX(const char* FilePath){
 //Try to open specyfied file; if unsuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"wb"); if(!FileHandler)return false;
 //Get image dimensions and write them to the file
 int wh[2]; //Create variable to hold width and height values of texture
 glBindTexture(GL_TEXTURE_2D,Name); //Bind this texture to make it current
 glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, wh);
 glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &wh[1]);
 fwrite(wh,4,2,FileHandler); //Write image's dimensions
 //If there is no image, stop here (indicating success anyway)
 if(wh[0]==0 || wh[1]==0){ fclose(FileHandler); return true; }
 //Calculate total size of image data
 unsigned long int size = wh[0]*wh[1]*4;
 //Create storage for pixels and get image's data
 unsigned char *pixels = new unsigned char[size];
 glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
 //Save pixels as tex-chunk to currently opened file
 fwrite(pixels,size,1,FileHandler);
 //Close file and destroy pixels location
 fclose(FileHandler);
 delete[] pixels;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
void TTexture::LoadFromChunk(FILE* CurrentlyOpenedFile){
 //Read image dimensions
 unsigned long int wh[2]; fread(wh,4,2,CurrentlyOpenedFile);
 //If there is no image, exit here (indicating success anyway)
 if(wh[0]==0 || wh[1]==0)return;
 //Calculate total size of image data
 unsigned long int size = wh[0]*wh[1]*4;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,CurrentlyOpenedFile);
 //Create, load, register texure
 glGenTextures(1,&Name);            //Create new texture name
 glBindTexture(GL_TEXTURE_2D,Name); //Make it current
 //Create mipmaps
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,
                   wh[0],wh[1],
                   GL_RGBA,GL_UNSIGNED_BYTE,pixels);
 //Destroy pixels location
 delete[] pixels;
}
//------------------------------------------------------------------------------
void TTexture::SaveToChunk(FILE* CurrentlyOpenedFile){
 //Get image dimensions and write them to the file
 int wh[2]; //Create variable to hold width and height values of texture
 glBindTexture(GL_TEXTURE_2D,Name); //Bind this texture to make it current
 glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, wh);
 glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &wh[1]);
 fwrite(wh,4,2,CurrentlyOpenedFile); //Write image's dimensions
 //If there is no image, stop here (indicating success anyway)
 if(wh[0]==0 || wh[1]==0)return;
 //Calculate total size of image data
 unsigned long int size = wh[0]*wh[1]*4;
 //Create storage for pixels and get image's data
 unsigned char *pixels = new unsigned char[size];
 glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
 //Save pixels as tex-chunk to currently opened file
 fwrite(pixels,size,1,CurrentlyOpenedFile);
 //Destroy pixels location
 delete[] pixels;
}
//------------------------------------------------------------------------------
void TTexture::PassChunk(FILE* CurrentlyOpenedFile){
 //Read image dimensions
 unsigned long int wh[2]; fread(wh,4,2,CurrentlyOpenedFile);

 //Pass pixels chunk
 fseek(CurrentlyOpenedFile, wh[0]*wh[1]*4, 1);
}
//------------------------------------------------------------------------------

//==============================================================================
#endif //End of header
